from choroplethcolourrange import ChoroplethColourRange
from infobox import Infobox
from coordinates import Coordinates
from choroplethcolourscheme import ChoroplethColourScheme
from mapdata import MapData
from svgmap import SVGMap
from from_shapefile import from_shapefile
from from_geojson import from_geojson
from from_csv import from_csv
